
import sys,os
import soclib

# import definitions from "segmentation.h"
import re
f = file('segmentation.h')
for line in f:
	res = re.match('#define\W+([^\W]+)\W+(.*)$', line)
	if res:
		exec(res.group(1) +' = ' + res.group(2))
f.close()

# parameters
useDma = True

# create architecture

arch = soclib.Architecture(
	cell_size = 4,
	plen_size = 9,
	addr_size = 32,
	rerror_size = 1,
	clen_size = 1,
	rflag_size = 1,
	srcid_size = 8,
	pktid_size = 1,
	trdid_size = 1,
	wrplen_size = 1)

arch.create('common:mapping_table', 'mapping_table',
	addr_bits  = [8],
	srcid_bits = [8],
	cacheability_mask=0x00300000)
arch.create('common:loader', 'loader', binary='soft/bin.soft')
vgmn = arch.create('caba:vci_vgmn', 'vgmn0', min_latency = 2, fifo_depth = 8)

xcache = arch.create('caba:vci_xcache_wrapper', 'mips0',
	ident = 0,
	icache_ways  = 1,
	icache_sets  = 8,
	icache_words = 4,
	dcache_ways  = 1,
	dcache_sets  = 8,
	dcache_words = 4,
	iss_t = 'common:mips32el',
)
vgmn.to_initiator.new() // xcache.vci

ram = arch.create('caba:vci_ram', 'ram0')
ram.addSegment('reset', RESET_BASE, RESET_SIZE, True)
ram.addSegment('excep', EXCEP_BASE, EXCEP_SIZE, True)
ram.addSegment('text',  TEXT_BASE,  TEXT_SIZE,  True)
ram.addSegment('data',  DATA_BASE,  DATA_SIZE,  True)
ram.vci // vgmn.to_target.new()

tty = arch.create('caba:vci_multi_tty', 'tty0', names=['tty0'])
tty.addSegment('tty0', TTY_BASE, TTY_SIZE, False)
tty.vci // vgmn.to_target.new()

sim = arch.create('caba:vci_simhelper', 'sim0')
sim.addSegment('sim0', SIMHELPER_BASE, SIMHELPER_SIZE, False)
sim.vci // vgmn.to_target.new()

fb0 = arch.create('caba:vci_framebuffer', 'fb0',
	width=FB_WIDTH, height=FB_HEIGHT)
fb0.addSegment('fb0', FB_BASE, FB_SIZE, False)
fb0.vci // vgmn.to_target.new()

if useDma:
	ip0 = arch.create('caba:vci_fir16_dma', 'ip0', burst_size=16)
	ip0.addSegment('ip0', IP_BASE, IP_SIZE, False)
	ip0.vci_target // vgmn.to_target.new()
	ip0.vci_initiator // vgmn.to_initiator.new()
	ip0.irq // xcache.irq[0]
else:
	ip0 = arch.create('caba:vci_fir16_slave', 'ip0')
	ip0.addSegment('ip0', IP_BASE, IP_SIZE, False)
	ip0.vci // vgmn.to_target.new()

arch.generate(soclib.PfDriver(outdir="."))

