/*
 * 
 * SOCLIB_LGPL_HEADER_BEGIN
 * 
 * This file is part of SoCLib, GNU LGPLv2.1.
 * 
 * SoCLib is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; version 2.1 of the License.
 * 
 * SoCLib is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with SoCLib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301 USA
 * 
 * SOCLIB_LGPL_HEADER_END
 * 
 * Copyright (c) CITI/INSA, 2009
 * 
 * Authors:
 * 	Ludovic L'Hours <ludovic.lhours@insa-lyon.fr>
 * 	Tanguy Risset <tanguy.risset@insa-lyon.fr>
 * 
 */

#ifndef FIX_INT_H
#define FIX_INT_H

template < unsigned int SCALE>
class fix_int {
private:
	signed long m_value;

public:
	fix_int(signed long value = 0, bool is_raw = false) {
		m_value = value;
		if (! is_raw) {
			m_value <<= SCALE;
		}
	}

	fix_int(const fix_int& a) {
		m_value = a.m_value;
	}

	fix_int& operator=(const fix_int& a) {
		m_value = a.m_value;
		return *this;
	}

	fix_int& operator=(signed short a) {
		m_value = a << SCALE;
		return *this;
	}

	fix_int operator*(const fix_int& a) {
		return fix_int((m_value * a.m_value)>>SCALE, true);
	}

	fix_int& operator+=(fix_int a) {
		m_value += a.m_value;
		return *this;
	}

	operator signed short() {
		return (signed short) (m_value>>SCALE);
	}

	template<unsigned int S>
	friend std::ostream& operator<<(std::ostream& out, const fix_int<S>& a);
};

template<unsigned int SCALE>
std::ostream& operator<<(std::ostream& out, const fix_int<SCALE>& a) {
	out << std::dec << (((double)a.m_value) / (1<<SCALE));
	return out;
}

#endif /* FIX_INT */

