/*
 * 
 * SOCLIB_LGPL_HEADER_BEGIN
 * 
 * This file is part of SoCLib, GNU LGPLv2.1.
 * 
 * SoCLib is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; version 2.1 of the License.
 * 
 * SoCLib is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with SoCLib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301 USA
 * 
 * SOCLIB_LGPL_HEADER_END
 * 
 * Copyright (c) CITI/INSA, 2009
 * 
 * Authors:
 * 	Ludovic L'Hours <ludovic.lhours@insa-lyon.fr>
 * 	Tanguy Risset <tanguy.risset@insa-lyon.fr>
 * 
 */

#ifndef VCI_FIR16_DMA
#define VCI_FIR16_DMA

#include <caba_base_module.h>
#include <generic_fifo.h>
#include <vci_target_fsm.h>
#include <vci_initiator_fsm.h>
#include <fir16_dma_regs.h>
#include <fix_int.h>

namespace soclib { namespace caba {

template < typename vci_param >
class VciFir16Dma : BaseModule {

	enum {
		FIFO_SIZE = 512,
	};

public:
    sc_in  < bool > p_clk;
    sc_in  < bool > p_resetn;

    soclib::caba::VciTarget<vci_param> p_vci_target;
    soclib::caba::VciInitiator<vci_param> p_vci_initiator;
    sc_out < bool > p_irq;

private:

	void transition ();
    void genMoore ();

	unsigned int m_from;
	unsigned int m_to;
	unsigned int m_length;
	unsigned int m_burst;

	bool m_dma_running;
	bool m_current_is_read;
	bool m_current_is_low;
	bool m_in_transfert;
	unsigned int m_input_data;
	unsigned int m_output_data;

	bool m_input_read_bool;
	bool m_input_write_bool;
	bool m_output_read_bool;
	bool m_output_write_bool;
	sc_dt::sc_uint<vci_param::B*8> m_input_write_data;
	sc_dt::sc_uint<vci_param::B*8> m_output_write_data;

	template< typename data_type >
	class XGenericFifo : public GenericFifo<data_type> {
	public:
		XGenericFifo(const char *name)
			: GenericFifo<data_type>(name, FIFO_SIZE) { }
	};

    XGenericFifo < sc_dt::sc_uint<vci_param::B*8> > m_input_fifo;
    XGenericFifo < sc_dt::sc_uint<vci_param::B*8> > m_output_fifo;

	soclib::caba::VciTargetFsm<vci_param, true> m_vci_target_fsm;
    soclib::caba::VciInitiatorFsm<vci_param> m_vci_init_fsm;

	fix_int<10> m_fir[16];
	fix_int<10> m_coeffs[16];

	bool on_write(int seg, typename vci_param::addr_t addr,
		typename vci_param::data_t data, int be);
	bool on_read(int seg, typename vci_param::addr_t addr,
		typename vci_param::data_t &data);

	void next_request();
	void next_read_request();
	void next_write_request();

	typedef typename vci_param::data_t data_t;
	bool on_write_fifo(data_t data);
	bool on_read_fifo(data_t& data);

	void read_request_done(VciInitiatorReq<vci_param> *req);
	void write_request_done(VciInitiatorReq<vci_param> *req);

	signed short doOneCycle(signed short data);

protected:
    SC_HAS_PROCESS (VciFir16Dma);
  
public:
    VciFir16Dma (sc_module_name insname, const MappingTable& mt,
			const IntTab& srcid, const IntTab& tgtid, int burst_size);

	~VciFir16Dma ();

};				

}} // end of soclib::caba

#endif /* VCI_FIR16_DMA */
