/*
 * 
 * SOCLIB_LGPL_HEADER_BEGIN
 * 
 * This file is part of SoCLib, GNU LGPLv2.1.
 * 
 * SoCLib is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; version 2.1 of the License.
 * 
 * SoCLib is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with SoCLib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301 USA
 * 
 * SOCLIB_LGPL_HEADER_END
 * 
 * Copyright (c) CITI/INSA, 2009
 * 
 * Authors:
 * 	Ludovic L'Hours <ludovic.lhours@insa-lyon.fr>
 * 	Tanguy Risset <tanguy.risset@insa-lyon.fr>
 * 
 */

#ifndef VCI_FIR16_SLAVE
#define VCI_FIR16_SLAVE

#include <caba_base_module.h>
#include <vci_target_fsm.h>
#include <fir16_slave_regs.h>
#include <fix_int.h>

namespace soclib { namespace caba {

template < typename vci_param >
class VciFir16Slave : BaseModule {

public:
    sc_in  < bool > p_clk;
    sc_in  < bool > p_resetn;

    soclib::caba::VciTarget<vci_param> p_vci;

private:

	void transition ();
    void genMoore ();

	soclib::caba::VciTargetFsm<vci_param, true> m_vci_target_fsm;

	fix_int<10> m_fir[16];
	fix_int<10> m_coeffs[16];

	signed short outData;

	bool on_write(int seg, typename vci_param::addr_t addr,
		typename vci_param::data_t data, int be);
	bool on_read(int seg, typename vci_param::addr_t addr,
		typename vci_param::data_t &data);

	signed short doOneCycle(signed short data);

protected:
    SC_HAS_PROCESS (VciFir16Slave);
  
public:
    VciFir16Slave (sc_module_name insname,
			const MappingTable& mt, const IntTab& tgtid);

	~VciFir16Slave ();

};				

}} // end of soclib::caba

#endif /* VCI_FIR16_SLAVE */
