/*
 * 
 * SOCLIB_LGPL_HEADER_BEGIN
 * 
 * This file is part of SoCLib, GNU LGPLv2.1.
 * 
 * SoCLib is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; version 2.1 of the License.
 * 
 * SoCLib is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with SoCLib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301 USA
 * 
 * SOCLIB_LGPL_HEADER_END
 * 
 * Copyright (c) CITI/INSA, 2009
 * 
 * Authors:
 * 	Ludovic L'Hours <ludovic.lhours@insa-lyon.fr>
 * 	Tanguy Risset <tanguy.risset@insa-lyon.fr>
 * 
 */

#include "vci_fir16_slave.h"

namespace soclib { namespace caba {

#define tmpl(x) \
template < typename vci_param > \
x VciFir16Slave<vci_param>

tmpl(bool)::on_write(int seg, typename vci_param::addr_t addr,
		typename vci_param::data_t data, int be) {

	unsigned int cell = addr / sc_dt::sc_uint<vci_param::N>(vci_param::B);

#ifdef SOCLIB_MODULE_DEBUG
	std::cout << name() << ": target write at " << std::hex << cell
			<< " data=" << data << " be=" << be << std::endl;
#endif

	if (cell >= FIR16_COEF0) {
		m_coeffs[cell - FIR16_COEF0] = fix_int<10>((signed long)data, true);
#ifdef SOCLIB_MODULE_DEBUG
		std::cout << name() << ": coeff[" << std::dec << (cell - FIR16_COEF0)
				<< "] = " << m_coeffs[cell-FIR16_COEF0] << std::endl;
#endif
		return true;
	}

	switch (cell) {

		case FIR16_INPUT:
			outData = doOneCycle(data);
			return true;
	}
	return false;
}

tmpl(bool)::on_read(int seg, typename vci_param::addr_t addr,
		typename vci_param::data_t &data) {

	unsigned int cell = addr / sc_dt::sc_uint<vci_param::N>(vci_param::B);

#ifdef SOCLIB_MODULE_DEBUG
	std::cout << name() << ": target read at address "
			<< std::hex << cell << std::endl;
#endif

	switch (cell) {

		case FIR16_OUTPUT:
			data = outData;
			return true;
	}
	return false;
}

tmpl(signed short)::doOneCycle(signed short data) {
	fix_int<10> sum = 0;
	fix_int<10> inData = data;
	for (int j = 15; j >= 0; --j) {
		sum += m_fir[j] * m_coeffs[j];
		m_fir[j] = j == 0 ? inData : m_fir[j-1];
	}
#ifdef SOCLIB_MODULE_DEBUG
	std::cout << name() << ": in = " << data
			<< "         out = " << sum << " | " << ((signed short)sum) << std::endl;
#endif
	return (signed short)sum;
}

tmpl(void)::transition () {

	if (p_resetn == false) {
		m_vci_target_fsm.reset();
		for (int i = 0; i < 16; ++i) {
			m_fir[i] = 0;
		}
		return;
	}

	m_vci_target_fsm.transition();

}

tmpl(void)::genMoore () {

	m_vci_target_fsm.genMoore();

}

tmpl(/**/)::VciFir16Slave(sc_module_name insname,
		const MappingTable& mt, const IntTab& tgtid)
	: BaseModule(insname),
	m_vci_target_fsm(p_vci, mt.getSegmentList(tgtid)) {

	m_vci_target_fsm.on_read_write(on_read, on_write);

	SC_METHOD (transition);
	dont_initialize();
	sensitive << p_clk.pos();

	SC_METHOD (genMoore);
	dont_initialize();
	sensitive << p_clk.neg();

}

tmpl(/**/)::~VciFir16Slave() {
}

}} // end of soclib::caba


/// Local Variables:
/// mode: hs-minor
/// c-basic-offset: 4
/// End:
