
#define USE_DMA

#include "stdio.h"
#include "stdlib.h"
#include "system.h"
#include "soclib/soclib_io.h"
#ifdef USE_DMA
#include "soclib/fir16_dma_regs.h"
#else
#include "soclib/fir16_slave_regs.h"
#endif
#include "soclib/simhelper.h"

#include "../segmentation.h"

static signed short image[256*256] = {
#include "image.dat"
};

static signed short dst_buffer[256*256];

#define SCALE 10
#define F(x, a) ((signed long)((x)<<(SCALE-(a))))
#define M(x) ((x)>>SCALE)

#define DO_BLUR

#ifdef DO_BLUR
static signed long fir_coeff[16] = {
		 0,  0,  0,  0,
		 F(1,3),  F(1,3),  F(1,3),  F(1,3),
		 F(1,3),  F(1,3),  F(1,3),  F(1,3),
		 0,  0,  0,  0 };
#else
static signed long fir_coeff[16] = {
		 0,  0,  0,  0,
		 0,  0,  0,  -F(1,1),
		 +F(1,1), 0, 0, 0,
		 0,  0,  0,  0 };
#endif

static signed long fir[16] = { 0, };

static void run_soft_filter(unsigned int size,
		signed short *from, signed short *to) {
	
	int i, j;

	for (i = 0; i < size; ++i) {
		signed long sum = 0;
		for (j = 15; j >= 0; --j) {
			sum += M(fir[j] * fir_coeff[j]);
			fir[j] = j == 0 ? F(from[i],0) : fir[j-1];
		}
		to[i] = M(sum);
	}
}

#ifdef USE_DMA
static int dma_terminated;

void dma_handler(int irq) {
	// deassert irq
	printf("DMA handler %d\n", irq);
	soclib_io_set(base(IP), FIR16_LENGTH, 0);
	dma_terminated = 1;
}

void run_hard_filter(unsigned int size,
		signed short *from, signed short *to) {
	
	int i;

	for (i = 0; i < 16; ++i) {
		soclib_io_set(base(IP), FIR16_COEF0 + i, fir_coeff[i]);
	}
	printf("Configure DMA\n");
	set_irq_handler(dma_handler);
    enable_hw_irq(0);
    irq_enable();
	dma_terminated = 0;
	soclib_io_set(base(IP), FIR16_FROM,   (unsigned int)from);
	soclib_io_set(base(IP), FIR16_TO,     (unsigned int)to);
	soclib_io_set(base(IP), FIR16_LENGTH, (unsigned int)size);

	printf("DMA started\n");

	while (! dma_terminated) {
		pause();
		// could do something else here
	}
	printf("DMA finished\n");
}
#else
void run_hard_filter(unsigned int size,
		signed short *from, signed short *to) {
	
	int i;

	for (i = 0; i < 16; ++i) {
		soclib_io_set(base(IP), FIR16_COEF0 + i, fir_coeff[i]);
	}

	for (i = 0; i < size; ++i) {
		soclib_io_set(base(IP), FIR16_INPUT, from[i]);
		to[i] = soclib_io_get(base(IP), FIR16_OUTPUT);
	}
}
#endif

int main(void) {

	uint8_t *fb = (uint8_t*) FB_BASE;
	unsigned int index = 0;
	const unsigned int width = 256;
	const unsigned int height= 256;
	const unsigned int size = width * height;

	unsigned int i, j;

	printf("Apply filter\n");
	//run_soft_filter(size, image, dst_buffer);
	run_hard_filter(size, image, dst_buffer);

	printf("cycles %d\n", run_cycles());

	printf("Copy image to frame buffer\n");
	index = 0;
	for (j = 0; j < height; ++j) {
		for (i = 0; i < width; ++i) {
#ifdef DO_BLUR
			*fb++ = dst_buffer[index++];
#else
			*fb++ = dst_buffer[index++] + 128;
#endif
		}
	}
	printf("End\n");

	exit(0);

	return 0;
}
